import React, { useCallback } from 'react';
import { normalizedAnimation } from '../AnimationInput.js';
import useAnimatedRef from '../hooks/useAnimatedRef.js';
import { mergeRefs } from './common.js';
const controlledAnimated = (props, forwardedRef) => {
    const { as: Tag = "div", animations, animationOptions, currentAnimation, onAnimationEnd, finishOnInterrupt = false, cancelOnInterrupt = false, commitStylesOnEnd = true, ...tagProps } = props;
    const decoratedOnEnd = (completedAnimationName, webAnimation) => {
        if (webAnimation) {
            if (commitStylesOnEnd) {
                try {
                    webAnimation.commitStyles();
                }
                catch (err) {
                    // element was unmounted
                }
            }
            if (webAnimation.playState !== 'finished') {
                if (finishOnInterrupt) {
                    webAnimation.finish();
                }
                else if (cancelOnInterrupt) {
                    webAnimation.cancel();
                }
            }
        }
        onAnimationEnd && onAnimationEnd(completedAnimationName, webAnimation);
    };
    const formattedAnimations = Object.fromEntries(Object.entries(animations).map(([name, anim]) => {
        const { keyframes, options } = normalizedAnimation(anim);
        return [name, {
                keyframes,
                options: { ...(animationOptions ? animationOptions : {}), ...options }
            }];
    }));
    const animatedRef = useAnimatedRef(currentAnimation, formattedAnimations, decoratedOnEnd);
    const elementRef = useCallback(mergeRefs(forwardedRef, animatedRef), [forwardedRef]);
    return (React.createElement(Tag, { ref: elementRef, ...tagProps }));
};
/**
 * An Animated component that is controlled by the currentAnimation prop
 * The component may use a different HTML tag delegate via the `as` prop
 * May accept a ref to forward to the HTML tag delegate.
 * Note: forwarding a ref returned from {@link hooks.useAnimatedRef} may result in undeterministic behavior
 *
 * @typeParam A the accepted animation names
 * @typeParam T the HTML Tag delegate
 */
const ControlledAnimated = React.forwardRef(controlledAnimated);
controlledAnimated.displayName = "ControlledAnimated";
export default ControlledAnimated;
