import { useState } from 'react';
import useQueuedState from './useQueuedState.js';
/**
 * Hook to useState that supports asynchronous transitions with distinct transition values
 *
 * State transitions may be queued via startTransition, and all queued transitions are completed in order by endTransition
 *
 * @typeParam T type of transition values
 * @typeParam S type of state
 * @param initialState
 * @param initialTransition if provided, initializes the state with currentTransition as initialTransition
 * @returns [state, startTransition, endTransition, currentTransition]: TransitioningState<S, T>
 */
function useTransitioningState(initialState, initialTransition = null) {
    const queuedState = useQueuedState(initialState);
    const [transition, setTransition] = useState(initialTransition);
    const endTransition = () => {
        setTransition(null);
        queuedState.transitionAll();
    };
    const startTransition = (nextState, nextTransition) => {
        queuedState.enqueue(nextState);
        // cancel transition if given null. Otherwise just setTransition
        if (!nextTransition) {
            endTransition();
        }
        else {
            setTransition(nextTransition);
        }
    };
    return [queuedState.current, startTransition, endTransition, transition];
}
export default useTransitioningState;
